/**************************************************************
 *  Systemy Mikroprocesorowe w Sterowaniu,  WEiTI PW 				  *
 *  ZL27ARM:PROJEKT_08 licznik TIM4, toggle LED1, LED2 				*
 **************************************************************/
#include "stm32f10x.h"
#include <stdbool.h> 		// true, false
#include <stdio.h>
#include ".\..\Drivers\KAmodKB4x4\KB_4x4.h"
#include ".\..\Drivers\LCD1602\lcd_1602.h"

// zmienne globalne do zapisywania znakow z klawiatury ----------
unsigned char key_buffer[32] = {0}; 		// deklaracja tablicy
volatile unsigned int key_pointer = 0;	// numer elementu w tablicy 'key_buffer'
unsigned char txt2LCD[32] = {0}; 				// bufor funkcji sprintf
unsigned char KB2mem[32] = {0}; 				// bufor funkcji sscanf
unsigned int RTC_run = 0;								// zezwala na wyswietlanie czasu

int  RCC_Config(void);
void GPIO_Config(void);
void NVIC_Config(void);
int  SysTick_Config_Div8(unsigned int Ticks);
void RTC_Config(void);
void TIM_Config(void);

int main(void) {
	//------------------- deklaracje zmiennych ------------------
	int g, m, s;
	unsigned char c;
	
  //------------------ konfiguracja systemu -------------------
  RCC_Config();   
  GPIO_Config(); 
  NVIC_Config();
	RTC_Config();
  GPIO_ResetBits(GPIOB, 0xFF00u);	// zgasimy wszystkie diody LED
	LCD_Initialize();              	//inicjalizacja wyswietlacza
	LCD_SendCommand(LCD_DISPLAY_ONOFF | LCD_DISPLAY_ON | LCD_CURSOR_ON);	
	SysTick_Config_Div8(9000000ul);
	TIM_Config();
	
	//------------------------ program C ------------------------
	LCD_WriteTextXY((unsigned char *)"wpisz\0", 0, 1);
	LCD_WriteTextXY((unsigned char *)"GG:MM:SS\0",8,1);
	LCD_GoTo(8,1);
	c = get_char();
	g = 10 * (c & 0x0F);
	c = get_char();
	g += (c & 0x0F);
	LCD_GoTo(11,1);
	c = get_char();
	m = 10 * (c & 0x0F);
	c = get_char();
	m += (c & 0x0F);
	LCD_GoTo(14,1);
	c = get_char();
	s = 10 * (c & 0x0F);
	c = get_char();
	s += (c & 0x0F);
	LCD_WriteTextXY((unsigned char *)"RTC    \0",0,0);
	RTC_SetCounter(3600*g + 60*m + s);
	RTC_run = 1;
	// petla nieskonczona
	s = 0; m = 0;
	while (1) {
		LCD_GoTo(s,1);
		LCD_WriteCharacter(m++ & 0x7F);
		delay_us(7900000);
		if(s++ > 14)
			s = 0;
	}
}
//------------ konfiguracja RCC: sygnaly taktujace ------------
int RCC_Config(void) {
  ErrorStatus HSEStartUpStatus;                          // zmienna opisujaca rezultat uruchomienia HSE

  RCC_DeInit();                                          // RCC na 'default reset state'
  RCC_HSEConfig(RCC_HSE_ON);                             // wlacz HSE
  HSEStartUpStatus = RCC_WaitForHSEStartUp();            // czekaj na gotowosc HSE
  if(HSEStartUpStatus == SUCCESS) {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);//
    FLASH_SetLatency(FLASH_Latency_2);                   // zwloka Flasha: 2 takty dla 72MHz
    RCC_HCLKConfig(RCC_SYSCLK_Div1);                     // HCLK=SYSCLK
    RCC_PCLK2Config(RCC_HCLK_Div1);                      // PCLK2=HCLK
    RCC_PCLK1Config(RCC_HCLK_Div2);                      // PCLK1=HCLK/2
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9); // PLLCLK = HSE*9 czyli 8MHz * 9 = 72 MHz
    RCC_PLLCmd(ENABLE);                                  // wlacz PLL
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);  // czekaj na uruchomienie PLL
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);           // ustaw PLL jako zrodlo sygnalu zegarowego
    while(RCC_GetSYSCLKSource() != 0x08);                // czekaj az PLL bedzie sygnalem zegarowym systemu
// konfiguracja sygnalow taktujacych uzywanych peryferii 
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE); // wlacz taktowanie portu AFIO (przerwania zewnetrzne)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);// wlacz taktowanie portu GPIO A (przyciski i wyjscia TIM1)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);// wlacz taktowanie portu GPIO B (LEDy i wyjscia TIM4)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);// wlacz taktowanie portu GPIO C (LCD1602)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD, ENABLE);// wlacz taktowanie portu GPIO D (klawiatura)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOE, ENABLE);// wlacz taktowanie portu GPIO E (buzzer)
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE); // wlacz taktowanie TIM4 z APB1
		
		return 0;
  }
	else
		return 1;
}
//------------------ konfiguracja portow GPIO -----------------
void GPIO_Config(void) {
  GPIO_InitTypeDef  GPIO_InitStructure; 
	
	// port A - Przyciski i joystick
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	// port B - diody LED i wyjscia TIM4
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;      // wyjscie w trybie push-pull
  GPIO_Init(GPIOB, &GPIO_InitStructure);
	
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;      // Alternate function push-pull
  GPIO_Init(GPIOB, &GPIO_InitStructure);
	
	// port D - klawiatura
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_OD;      // wyjscie w trybie Open-Drain
  GPIO_Init(GPIOD, &GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;	// wejscie w trybie FLOATING,
  GPIO_Init(GPIOD, &GPIO_InitStructure);
	GPIO_ResetBits(GPIOD, GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13);
	GPIO_AFIODeInit();
	
	// port E - buzzer
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0; 	// pin PE0
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;      // wyjscie w trybie push-pull
  GPIO_Init(GPIOE, &GPIO_InitStructure);
}

// ----------- konfiguracja kontrolera przerwan NVIC ----------
void NVIC_Config(void) {
	NVIC_InitTypeDef  NVIC_InitStructure; 
  EXTI_InitTypeDef  EXTI_InitStructure; 
	
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);	// tablica wektorow przerwan we FLASH
	// Konfiguracja NVIC - ustawienia priorytetow przerwania EXTI0
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);                //Wybor modelu grupowania przerwan
	
  // linia 6 = COL1 -------------------------------------------------------------------------
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOD, GPIO_PinSource6);    //Ustawienie zrodla przerwania
  //Konfiguracja przerwania EXTI na linie 6
  EXTI_InitStructure.EXTI_Line = EXTI_Line6;                     //Wybor linii 
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;            //Ustawienie generowania przerwania (a nie zdarzenia)
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;        //Wyzwalanie zboczem opadajacym (wcisniecie przycisku)
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;                      //Wlaczenie przerwania
  EXTI_Init(&EXTI_InitStructure);
	// linia 7 = COL2 -------------------------------------------------------------------------
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOD, GPIO_PinSource7);    //Ustawienie zrodla przerwania
  //Konfiguracja przerwania EXTI na linie 7
  EXTI_InitStructure.EXTI_Line = EXTI_Line7;                     //Wybor linii 
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;            //Ustawienie generowania przerwania (a nie zdarzenia)
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;        //Wyzwalanie zboczem opadajacym (wcisniecie przycisku)
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;                      //Wlaczenie przerwania
  EXTI_Init(&EXTI_InitStructure);
	// linia 8 = COL3 ---------------------------------------------------------------------------
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOD, GPIO_PinSource8);    //Ustawienie zrodla przerwania
  //Konfiguracja przerwania EXTI na linie 8
  EXTI_InitStructure.EXTI_Line = EXTI_Line8;                     //Wybor linii 
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;            //Ustawienie generowania przerwania (a nie zdarzenia)
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;        //Wyzwalanie zboczem opadajacym (wcisniecie przycisku)
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;                      //Wlaczenie przerwania
  EXTI_Init(&EXTI_InitStructure);
	// linia 9 = COL4 ---------------------------------------------------------------------------
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOD, GPIO_PinSource9);    //Ustawienie zrodla przerwania
  //Konfiguracja przerwania EXTI na linie 9
  EXTI_InitStructure.EXTI_Line = EXTI_Line9;                     //Wybor linii 
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;            //Ustawienie generowania przerwania (a nie zdarzenia)
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;        //Wyzwalanie zboczem opadajacym (wcisniecie przycisku)
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;                      //Wlaczenie przerwania
  EXTI_Init(&EXTI_InitStructure);
	
	// Wybor konfigurowanego IRQ ----------------------------------------------------------------
	NVIC_InitStructure.NVIC_IRQChannel = EXTI9_5_IRQn;             //Wybor konfigurowanego IRQ
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 3;      //Priorytet grupowy
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;             //Podpriorytet
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;                //Wlaczenie obslugi IRQ
  NVIC_Init(&NVIC_InitStructure);
	
	// Wlacz przerwanie od RTC ------------------------------------------------------------------
  NVIC_InitStructure.NVIC_IRQChannel = RTC_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 2;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}
// --------- konfiguracja licznika systemowego SysTick --------
int SysTick_Config_Div8(unsigned int Ticks){ 
  unsigned int ctrl;

  if (Ticks > SysTick_LOAD_RELOAD_Msk)  return 1;             	// czy wartosc poczatkowa nie przekracza max
                                                                
  SysTick->LOAD = (Ticks & SysTick_LOAD_RELOAD_Msk) - 1;        // ustaw wartosc poczatkowa licznika 
  NVIC_SetPriority (SysTick_IRQn, 0);                           // ustaw priorytet przerwania
  SysTick->VAL  = 0;                                            // ustaw wartosc aktualna licznika
  ctrl=SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk;      // ustaw flagi wlaczenia SysTick IRQ  i samego licznika
	ctrl &= SysTick_CLKSource_HCLK_Div8;
  SysTick->CTRL = ctrl;                  // zapisz ustawienia do rejestru sterujacego SysTick (i wlacz licznik)
  return 0;
}
 // -------------------  konfigurowanie RTC -------------------
void RTC_Config(void) {
	RCC -> APB1ENR |= RCC_APB1Periph_PWR;	// Power  interface clock enable
	RCC -> APB1ENR |= RCC_APB1Periph_BKP; // Backup interface clock enable

	PWR -> CR |= PWR_CR_DBP;				// Disable backup domain write protection
	// Select LSE clock
	RCC -> CSR &= ~RCC_CSR_LSION; 	// LSI (w RCC) off
	RCC -> BDCR &= ~RCC_BDCR_LSEBYP;// External Low Speed oscillator Bypass = off
	RCC -> BDCR |= RCC_BDCR_RTCSEL_LSE;	/*!< LSE oscillator clock used as RTC clock */
	RCC -> BDCR |= RCC_BDCR_LSEON;			/*!< External Low Speed oscillator enable */
	while((RCC -> BDCR & RCC_BDCR_LSERDY) == 0)
		;																	// czekamy na gotowosc LSE
	RCC -> BDCR |= RCC_BDCR_RTCEN;			/*!< RTC clock enable */
	
  RTC_WaitForSynchro();                    //Oczekiwanie na synchronizacje
  RTC_WaitForLastTask();                   //Oczekiwanie az rejestry RTC zostana zapisane
  RTC_SetPrescaler(32767);                 //Okres RTC bedzie wynosil 1s  RTC period = RTCCLK/RTC_PR = (32.768 KHz)/(32767+1)
  RTC_WaitForLastTask();
  RTC_ITConfig(RTC_IT_SEC, ENABLE);        // Wlacz przerwanie "sekundnika" RTC 
  RTC_WaitForLastTask();
  RCC_ClearFlag();
}
 // --------------  konfigurowanie licznika TIM4 --------------
void TIM_Config(void) {
  //Konfiguracja timerow
  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
  TIM_OCInitTypeDef  TIM_OCInitStructure;
	
  //Konfiguracja licznika 4: ustawienia taktowania i trybu pracy licznika 4
	TIM_InternalClockConfig(TIM4);	// zeruje bity SMS w rejestrze TIM4_SMCR, w polaczeniu z CEN(TIM4_CR1)=1 daje CK_INT
	TIM_TimeBaseStructure.TIM_Prescaler = 3600-1;               //taktowanie licznka fclk = 72MHz/3600 = 20kHz
	TIM_TimeBaseStructure.TIM_Period = 20000-1;                 //okres przepelnien licznika = 20000 taktow = 1 sekunda
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;    	//dzielnik zegara dla ukladu generacji dead-time i filtra
 	TIM_TimeBaseStructure.TIM_RepetitionCounter=0;          		//licznik powtorzen
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;	//tryb pracy licznika
	TIM_TimeBaseInit(TIM4, &TIM_TimeBaseStructure);            	//Inicjalizacja licznika
	
	//Konfiguracja kanalu 3
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Toggle;       	//tryb pracy kanalu
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;//wlaczenie generowania sygnalu na wyjsciu licznika
	TIM_OCInitStructure.TIM_Pulse =12000;                      //przesuniecie w czasie 10000 taktow = 1 sekunda
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;   //polaryzacja wyjscia
	TIM_OC3Init(TIM4, &TIM_OCInitStructure);                    //Inicjalizacja kanalu 3 licznika TIM4	
	TIM_OC3PreloadConfig(TIM4, TIM_OCPreload_Enable);           //zapis ustawien, wlaczenie buforowania  - zmiany konfiguracji
																															//  uwzgledniane sa przy nastepnym update
	//Konfiguracja kanalu 4
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;         	//tryb pracy kanalu
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;//wlaczenie generowania sygnalu na wyjsciu licznika
	TIM_OCInitStructure.TIM_Pulse =4000;                        // wypelnienie 20%
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;   //polaryzacja wyjscia
	TIM_OC4Init(TIM4, &TIM_OCInitStructure);                    //Inicjalizacja kanalu 4 licznika TIM4	
	TIM_OC4PreloadConfig(TIM4, TIM_OCPreload_Enable);           //zapis ustawien, wlaczenie buforowania  - zmiany konfiguracji
																															//  uwzgledniane sa przy nastepnym update
  TIM_Cmd(TIM4, ENABLE);																			// Wlaczenie timera TIM4	
}
	
/**************** (C) COPYRIGHT 2016 PLUMicroelectronics *****END OF FILE******/



